use std::ops::DerefMut;
use std::sync::Mutex;

use eframe::egui::{self};

use plate_tool_lib::plate::PlateFormat;

use crate::plate::{add_grid, PlateUiState};
use crate::transfer_menu::{self, transfer_menu, TransferMenuState};

#[non_exhaustive]
struct MainWindowState {
    show_side_panel: bool,
}

impl Default for MainWindowState {
    fn default() -> Self {
        Self {
            show_side_panel: true,
        }
    }
}

#[non_exhaustive]
pub struct PlateToolEframe {
    format: PlateFormat,
    source_plate_state: Mutex<PlateUiState>,
    destination_plate_state: Mutex<PlateUiState>,
    main_window_state: MainWindowState,
    transfer_menu_state: TransferMenuState,
}

impl Default for PlateToolEframe {
    fn default() -> Self {
        Self {
            format: PlateFormat::W96,
            source_plate_state: Mutex::new(PlateUiState::default()),
            destination_plate_state: Mutex::new(PlateUiState::default()),
            main_window_state: MainWindowState::default(),
            transfer_menu_state: TransferMenuState::default(),
        }
    }
}

impl PlateToolEframe {
    pub fn new(cc: &eframe::CreationContext<'_>) -> Self {
        // Would load state here

        Default::default()
    }
}

impl eframe::App for PlateToolEframe {
    // State storage
    /*
    fn save(&mut self, storage: &mut dyn eframe::Storage) {
        unimplemented!()
    };
    */

    fn update(&mut self, ctx: &eframe::egui::Context, _frame: &mut eframe::Frame) {
        egui::TopBottomPanel::top("top_panel").show(ctx, |ui| {
            egui::menu::bar(ui, |ui| {
                ui.menu_button("File", |ui| {
                    if ui.button("New").clicked() {
                    }
                    ui.menu_button("Export", |ui| {
                        if ui.button("Export as CSV").clicked() {
                        }
                        if ui.button("Export as JSON").clicked() {
                        }
                    });
                    ui.menu_button("Import", |ui| {
                        if ui.button("Import from JSON").clicked() {
                        }
                        if ui.button("Import transfer from CSV").clicked() {
                        }
                    });
                });
                ui.menu_button("Options", |ui| {
                    ui.menu_button("Styles", |ui| {
                        if ui.button("Toggle transfer hashes").clicked() {
                        }
                        if ui.button("Toggle volume heatmap").clicked() {
                        }
                        if ui.button("Toggle current coordinates view").clicked() {
                        }
                    });
                    ui.menu_button("Exports", |ui| {
                        if ui.button("Change CSV export type").clicked() {
                        }
                    });
                    ui.menu_button("Windows", |ui| {
                        ui.toggle_value(&mut self.main_window_state.show_side_panel, "Toggle side panel");
                    });
                });
            });
        });

        if self.main_window_state.show_side_panel {
            egui::SidePanel::left("side_menus").show(ctx, |ui| {
                ui.vertical(|ui| {
                    transfer_menu(ui, &mut self.transfer_menu_state);
                });
            });
        }

        egui::CentralPanel::default().show(ctx, |ui| {
            ui.vertical(|ui| {
                let available_size = ui.available_size();
                let half_height = {
                    let mut x = available_size;
                    x.y /= 2.0;
                    x
                };
                add_grid(
                    half_height,
                    PlateFormat::W1536,
                    ui,
                    self.source_plate_state.lock().unwrap().deref_mut(),
                );
                add_grid(
                    half_height,
                    PlateFormat::W1536,
                    ui,
                    self.destination_plate_state.lock().unwrap().deref_mut(),
                );
            });
        });
    }
}
